#!/usr/bin/env Rscript

# force sorting
if (capabilities("ICU")) {
  icuSetCollate(locale = "en_US.UTF-8")
} else {
  print("Can not force sorting, this could result in unpredicted results.")
}

# following modules will be ignored
ignore <- c("modules/data.mining")

files <-
  c(
    list.files(
      path = "base",
      full.names = TRUE,
      pattern = "^DESCRIPTION$",
      recursive = TRUE
    ),
    list.files(
      path = "modules",
      full.names = TRUE,
      pattern = "^DESCRIPTION$",
      recursive = TRUE
    ),
    list.files(
      path = "models",
      full.names = TRUE,
      pattern = "^DESCRIPTION$",
      recursive = TRUE
    )
  )

## Required dependencies
pecan <- c()
depends <- c()
docker <- c()
remotes <- c()
d <- purrr::walk(
  files,
  function(x) {
    f <- dirname(x)
    if (f %in% ignore) return()

    # load DESCRIPTION file
    d <- desc::desc(file = x)
    deps <- d$get_deps()[["package"]]

    # PEcAn dependencies
    y <- deps[grepl("^PEcAn.", deps)]
    p <- d$get_field("Package")
    pecan[[p]] <<- f
    depends[[f]] <<- y

    # Dockerfile dependencies
    z <- y["package"]
    z <- deps[!grepl("^PEcAn.", deps)]
    docker <<- unique(c(docker, z))

    # Dockerfile remote dependencies
    if (!purrr::is_empty(d$get_remotes())) {
      deps <- d$get_remotes()
      github <- gsub("github::", "", deps[grepl("^github::", deps)])
      remotes <<- unique(c(remotes, github))

      bad <- deps[!grepl("^github::", deps)]
      if (!purrr::is_empty(bad)) {
       print(bad)
      }
    }
  }
)

# write for makefile
cat("# autogenerated", file = "Makefile.depends", sep = "\n", append = FALSE)
for (name in names(depends)) {
  x <- paste0("$(call depends,", name, "): |")
  for (p in depends[[name]]) {
    # TEMP HACK: Don't declare known circular deps in utils Suggests
    if (name == "base/utils" && p == "PEcAn.DB") next
    x <- paste0(x, " .install/", pecan[[p]])
  }
  cat(x, file = "Makefile.depends", sep = "\n", append = TRUE)
}

# write for docker dependency image
cat("#!/usr/bin/env Rscript",
    "# autogenerated do not edit",
    "# use scripts/generate_dependencies.R",
    "",
    "# Don\'t use X11 for rgl",
    "Sys.setenv(RGL_USE_NULL = TRUE)",
    "rlib <- Sys.getenv('R_LIBS_USER', '/usr/local/lib/R/site-library')",
    "Sys.setenv(RLIB = rlib)",
    "",
    "# install remotes first in case packages are references in dependencies",
    "lapply(c(",
    paste(shQuote(sort(remotes)), collapse = ",\n"),
    "), remotes::install_github, lib = rlib)",
    "",
    "# install all packages (depends, imports, suggests)",
    "wanted <- c(", paste(shQuote(sort(docker)), sep = "", collapse = ",\n"), ")",
    "missing <- wanted[!(wanted %in% installed.packages()[,'Package'])]",
    "install.packages(missing, lib = rlib)",
    file = "docker/depends/pecan.depends.R", sep = "\n", append = FALSE)
